#line 1 "/builddir/build/BUILD/cppcheck-2.16.2/lib/vf_debug.cpp"
/*
 * Cppcheck - A tool for static C/C++ code analysis
 * Copyright (C) 2007-2024 Cppcheck team.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "vf_debug.h"

#include "errorlogger.h"
#include "errortypes.h"
#include "settings.h"
#include "token.h"
#include "tokenlist.h"
#include "vfvalue.h"

#include "vf_common.h"

#include <list>
#include <string>

namespace ValueFlow
{
    void analyzeDebug(TokenList& tokenlist, ErrorLogger& errorLogger, const Settings& settings)
    {
        if (!settings.debugnormal && !settings.debugwarnings)
            return;
        for (Token* tok = tokenlist.front(); tok; tok = tok->next()) {
            if (tok->getTokenDebug() != TokenDebug::ValueFlow)
                continue;
            if (tok->astParent() && tok->astParent()->getTokenDebug() == TokenDebug::ValueFlow)
                continue;
            for (const Value& v : tok->values()) {
                std::string msg = "The value is " + debugString(v);
                ErrorPath errorPath = v.errorPath;
                errorPath.insert(errorPath.end(), v.debugPath.cbegin(), v.debugPath.cend());
                errorPath.emplace_back(tok, "");
                errorLogger.reportErr({errorPath, &tokenlist, Severity::debug, "valueFlow", msg, CWE{0}, Certainty::normal});
            }
        }
    }
}
